<?php
/**
 * An object oriented wrapper for the string type in php.
 * 
 * @author      José Miguel Molina <hi@mvader.me>
 * @copyright   2013 José Miguel Molina
 * @link        https://github.com/mvader/php-string
 * @license     https://raw.github.com/mvader/php-string/master/LICENSE
 * @version     1.0.0
 * @package     php-string
 *
 * MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * String
 *
 * @package php-string
 * @author José Miguel Molina
 * @since 0.0.1
 */
class String implements \ArrayAccess, \IteratorAggregate
{
    /**
     * @var string Internal php string representation of the String\String object.
     */
    private $str = '';

    /**
     * Constructor
    * @param string $string The initial value for the String\String object.
     */
    public function __construct($string)
    {
        $this->append($string);
    }

    /**
     * Checks for the existance of the given index in the string.
     * @param int $index The desired index.
     * @return bool
     */
    public function offsetExists($index)
    {
        if (!is_int($index))
            return false;

        return ($index >= 0 && $index < $this->length());
    }

    /**
     * Returns the character at the given index.
     * @param int $index The desired index.
     * @return string|bool The character at the given index or false if the index doesn't exist.
     */
    public function offsetGet($index)
    {
        if (!$this->offsetExists($index))
            return false;
        return $this->str[$index];
    }

    /**
     * Replaces the character at the given index.
     * @param int $index The desired index.
     * @param string $value Replacement character.
     * @throws Exception
     * @todo Throw custom exception.
     */
    public function offsetSet($index, $value)
    {
        if (is_string($value) && strlen($value) === 1) {
            if (!is_null($index))
                $this->str[$index] = $value;
            else
                $this->append($value);
        } else {
            throw new Exception();
        }
    }

    /**
     * Erases the character at the given index.
     * @param int $index The desired index.
     */
    public function offsetUnset($index)
    {
        if ($offset - 1 == $this->length())
            $this->str = $this->substring(0, $index);
        else
            $this->str = $this->substring(0, $index) . 
                         $this->substring($index + 1);
    }

    /**
     * IteratorAggregate : getIterator
     */
    public function getIterator()
    {
        return new ArrayIterator($this->toArray());
    }

    /**
     * Appends a string to the current value.
     * @param string $string The string to append.
     */
    public function append($string)
    {
        $this->str .= (string) $string;
    }

    /**
     * This function returns a string with whitespace stripped from the beginning and end of the string.
     * @link http://www.php.net/manual/en/function.trim.php PHP documentation for trim.
     * @param string $charList A list with all characters that you want to be stripped
     * @param bool $alter If true the current string value will be altered, otherwhise it will just be returned but it won't change.
     * @return string If $alter is false.
     */
    public function trim($charList = false, $alter = false)
    {
        if ($alter)
        {
            if ($charList === false)
                $this->str = trim($this->str);
            else
                $this->str = trim($this->str, $charList);
        }
        else
        {
            if ($charList === false)
                return trim($this->str);
            else
                return trim($this->str, $charList);
        }
    }

    /**
     * This function returns a string with whitespace stripped from the beginning of the string.
     * @link http://www.php.net/manual/en/function.ltrim.php PHP documentation for ltrim.
     * @param string $charList A list with all characters that you want to be stripped
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function leftTrim($charList = false, $alter = false)
    {
        if ($alter)
        {
            if ($charList === false)
                $this->str = ltrim($this->str);
            else
                $this->str = ltrim($this->str, $charList);
        }
        else
        {
            if ($charList === false)
                return ltrim($this->str);
            else
                return ltrim($this->str, $charList);
        }
    }

    /**
     * This function returns a string with whitespace stripped from the end of the string.
     * @link http://www.php.net/manual/en/function.rtrim.php PHP documentation for rtrim.
     * @param string $charList A list with all characters that you want to be stripped
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function rightTrim($charList = false, $alter = false)
    {
        if ($alter)
        {
            if ($charList === false)
                $this->str = rtrim($this->str);
            else
                $this->str = rtrim($this->str, $charList);
        }
        else
        {
            if ($charList === false)
                return rtrim($this->str);
            else
                return rtrim($this->str, $charList);
        }
    }

    /**
     * Split a string into smaller chunks.
     * @link http://www.php.net/manual/en/function.chunk-split.php PHP reference for chunk_split.
     * @param int $chunkLen The chunk length.
     * @param string $end The line ending sequence.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function chunkSplit($chunkLen = 76, $end = "\r\n", $alter = false)
    {
        if ($alter)
            $this->str = chunk_split($this->str, $chunkLen, $end);
        else
            return chunk_split($this->str, $chunkLen, $end);
    }

    /**
     * Return information about characters used in a string.
     * @link http://www.php.net/manual/en/function.count-chars.php PHP documentation for count_chars.
     * @param int $mode See php documentation for count_chars.
     * @return mixed Depends on mode. See php documentation for count_chars.
     */
    public function countChars($mode = 0)
    {
        return count_chars($this->str, $mode);
    }

    /**
     * Calculates the crc32 polynomial of a string
     * @link http://www.php.net/manual/en/function.crc32.php PHP documentation for crc32.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function crc32($alter = false)
    {
        if ($alter)
            $this->str = crc32($this->str);
        else
            return crc32($this->str);
    }

    /**
     * It will return a hashed string using the standard Unix DES-based algorithm or alternative algorithms that may be available on the system.
     * @link http://www.php.net/manual/en/function.crypt.php PHP documentation for crypt.
     * @param string $salt An optional salt string to base the hashing on. If not provided, the behaviour is defined by the algorithm implementation and can lead to unexpected results.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function crypt($salt = '', $alter = false)
    {
        if ($alter)
            $this->str = crypt($this->str, $salt);
        else
            return $this->str = crypt($this->str, $salt);
    }

    /**
     * Returns an array of strings, each of which is a substring of string formed by splitting it on boundaries formed by the string delimiter.
     * @link http://www.php.net/manual/en/function.explode.php PHP documentation for explode.
     * @param string $delimiter The boundary string.
     * @param int $limit If limit is set and positive, the returned array will contain a maximum of limit elements with the last element containing the rest of string. If the limit parameter is negative, all components except the last -limit are returned. If the limit parameter is zero, then this is treated as 1.See php documentation for explode.
     * @return array Returns an array of strings created by splitting the string parameter on boundaries formed by the delimiter. See php documentation for explode.
     */
    public function split($delimiter = ',', $limit = false)
    {
        $args = array($delimiter, $this->str);
        if ($limit !== false)
            $args[] = $limit;
        return call_user_func_array('explode', $args);
    }

    /**
     * Returns a one-character string containing the character specified by ascii.
     * @param int $char The ASCII code.
     * @return string The character for the given ASCII code.
     */
    public static function char($char)
    {
        return chr((int) $char);
    }

    /**
     * Returns the ASCII value of the first character of the string.
     * @param string $char The character.
     * @return int The ASCII for the given character.
     */
    public static function ascii($char)
    {
        return ord($char);
    }

    /**
     * Parses string as if it were the query string passed via a URL and sets variables in the current scope.
     * @link http://www.php.net/manual/en/function.parse-str.php PHP documentation for parse_str.
     * @param string $str The input string.
     * @param array $array If the second parameter is present, variables are stored in this variable as array elements instead.
     */
    public static function parseStr($str, &$array = null)
    {
        if (is_null($array))
            parse_str($str);
        else
            parse_str($str, $array);
    }

    /**
     * Returns an ASCII string containing the hexadecimal representation of the string. The conversion is done byte-wise with the high-nibble first.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string|bool If $alter is false or FALSE on failure.
     */
    public function binToHex($alter = false)
    {
        if ($alter) {
            $result = bin2hex($this->str);
            if ($result)
                $this->str = $result;
            else
                return false;
        } else
            return bin2hex($this->str);
    }

    /**
     * Decodes a hexadecimally encoded binary string.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string|bool If $alter is false or FALSE on failure.
     */
    public function hexToBin($alter = false)
    {
        if ($alter) {
            $result = hex2bin($this->str);
            if ($result)
                $this->str = $result;
            else
                return false;
        } else
            return hex2bin($this->str);
    }

    /**
     * Convert all applicable characters to HTML entities.
     * @link http://www.php.net/manual/en/function.htmlentities.php PHP documentation for htmlentities.
     * @param int $flags A bitmask of one or more of the following flags, which specify how to handle quotes, invalid code unit sequences and the used document type. The default is ENT_COMPAT | ENT_HTML401.
     * @param string $encoding Defines encoding used in conversion. If omitted, the default value for this argument is ISO-8859-1 in versions of PHP prior to 5.4.0, and UTF-8 from PHP 5.4.0 onwards. 
     * @param bool $doubleEncode When is turned off PHP will not encode existing html entities. The default is to convert everything.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function htmlEntities(
        $flags = null,
        $encoding = 'UTF-8',
        $doubleEncode = true,
        $alter = false
    ) {
        if (is_null($flags))
            $flags = (ENT_COMPAT | ENT_HTML401);

        if ($alter)
            $this->str = htmlentities(
                $this->str,
                $flags,
                $encoding,
                $doubleEncode
            );
        else
            return htmlentities(
                $this->str,
                $flags,
                $encoding,
                $doubleEncode
            );
    }

    /**
     * Convert special characters to HTML entities.
     * @link http://www.php.net/manual/en/function.htmlspecialchars.php PHP documentation for htmlspecialchars.
     * @param int $flags A bitmask of one or more of the following flags, which specify how to handle quotes, invalid code unit sequences and the used document type. The default is ENT_COMPAT | ENT_HTML401.
     * @param string $encoding Defines encoding used in conversion. If omitted, the default value for this argument is ISO-8859-1 in versions of PHP prior to 5.4.0, and UTF-8 from PHP 5.4.0 onwards. 
     * @param bool $doubleEncode When is turned off PHP will not encode existing html entities. The default is to convert everything.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function htmlSpecialChars(
        $flags = null,
        $encoding = 'UTF-8',
        $doubleEncode = true,
        $alter = false
    ) {
        if (is_null($flags))
            $flags = (ENT_COMPAT | ENT_HTML401);

        if ($alter)
            $this->str = htmlspecialchars(
                $this->str,
                $flags,
                $encoding,
                $doubleEncode
            );
        else
            return htmlspecialchars(
                $this->str,
                $flags,
                $encoding,
                $doubleEncode
            );
    }

    /**
     *  Convert special HTML entities back to characters.
     * @link http://www.php.net/manual/en/function.htmlspecialchars-decode.php PHP documentation for htmlspecialchars_decode.
     * @param int $flags A bitmask of one or more of the following flags, which specify how to handle quotes, invalid code unit sequences and the used document type. The default is ENT_COMPAT | ENT_HTML401.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function htmlSpecialCharsDecode(
        $flags = null,
        $alter = false
    ) {
        if (is_null($flags))
            $flags = (ENT_COMPAT | ENT_HTML401);

        if ($alter)
            $this->str = htmlspecialchars_decode($this->str, $flags);
        else
            return htmlspecialchars_decode($this->str, $flags);
    }

    /**
     * Convert all HTML entities to their applicable characters.
     * @link http://www.php.net/manual/en/function.html-entity-decode.php PHP documentation for html_entity_decode.
     * @todo implement.
     */
    public function htmlEntitiesDecode()
    {

    }

    /**
     * Join array elements with a string.
     * @link http://www.php.net/manual/en/function.implode.php PHP documentation for implode.
     * @param string $glue Separator string. Defaults to an empty string.
     * @param array $pieces The array of strings to join.
     * @param bool $alter If true the current string value will be replaced.
     * @param bool $return If true the result will be returned.
     * @return string If $return is set to true.
     */
    public function join(
        $glue = '',
        array $pieces,
        $alter = false,
        $return = false
    ) {
        $result = implode($glue, $pieces);
        if ($alter)
            $this->str = $result;

        if ($return)
            return $result;
    }

    /**
     * Make a string's first character lowercase.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function lowerFirst($alter = false)
    {
        if ($alter)
            $this->str = lcfirst($this->str);
        else
            return lcfirst($this->str);
    }

    /**
     * Calculate the md5 hash of a string.
     * @param bool $rawOutput If it's set to true the md5 digest is instead returned in raw binary format with a length of 16.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function md5($rawOutput = false, $alter = false)
    {
        if ($alter)
            $this->str = md5($this->str, $rawOutput);
        else
            return md5($this->str, $rawOutput);
    }

    /**
     * Calculate the sha1 hash of a string.
     * @param bool $rawOutput If it's set to true the sha1 digest is instead returned in raw binary format with a length of 20, otherwise the returned value is a 40-character hexadecimal number.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function sha1($rawOutput = false, $alter = false)
    {
        if ($alter)
            $this->str = sha1($this->str, $rawOutput);
        else
            return sha1($this->str, $rawOutput);
    }

    /**
     * Inserts HTML line breaks before all newlines in a string.
     * @param bool $isXhtml Whether to use XHTML compatible line breaks or not.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function nlToBr($isXhtml = true, $alter = false)
    {
        if ($alter)
            $this->str = nl2br($this->str, $isXhtml);
        else
            return nl2br($this->str, $isXhtml);
    }

    /**
     * Output a formatted string.
     * @param mixed args The arguments to format the current string value with.
     */
    public function printf()
    {
        $args = array_merge(array($this->str), func_get_args());
        call_user_func_array('printf', $args);
    }

    /**
     * Calculate the similarity between two strings-
     * @param string|String $string The string to compare the current string value to.
     * @param float $percent By passing a reference as second argument, the method will calculate the similarity in percent for you.
     * @return int The number of matching chars in both strings.
     */
    public function similarText($string, &$percent = null)
    {
        if (is_null($percent))
            $percent = 0.0;
        return similar_text($this->str, (string) $string, $percent);
    }

    /**
     * Formats the current string value.
     * @link http://www.php.net/manual/en/function.sprintf.php PHP documentation for sprintf.
     * @param mixed args The arguments to format the current string value with.
     * @return string The formatted string.
     */
    public function format()
    {
        $args = array_merge(array($this->str), func_get_args());
        return call_user_func_array('sprintf', $args);
    }

    /**
     * Replace all occurrences of the search string with the replacement string.
     * @param mixed $search The value being searched for, otherwise known as the needle. An array may be used to designate multiple needles.
     * @param mixed $replace The replacement value that replaces found search values. An array may be used to designate multiple replacements.
     * @param int $count If passed, this will be set to the number of replacements performed.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function replace($search, $replace, &$count = null, $alter = false)
    {
        if ($alter)
            $this->str = str_replace($search, $str, $this->str, $count);
        else
            return str_replace($search, $str, $this->str, $count);
    }

    /**
     * Randomly shuffles a string.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function shuffle($alter = false)
    {
        if ($alter)
            $this->str = str_shuffle($this->str);
        else
            return str_shuffle($this->str);
    }

    /**
     * Convert a string to an array
     * @param int $itemLength Length of the array items.
     * @return array The string converted to an array.
     */
    public function toArray($itemLength = 1)
    {
        return str_split($this->str, $itemLength);
    }

    /**
     * Return information about words used in a string.
     * @link http://www.php.net/manual/en/function.str-word-count.php PHP documentation for str_word_count.
     * @param int $format Specify the return value of this function. See php documentation for more information.
     * @param string $charList A list of additional characters which will be considered as 'word'.
     * @return int|array Returns an array or an integer, depending on the format chosen.
     */
    public function wordCount($format = 0, $charList = '')
    {
        return str_word_count($this->str, $format, $charList);
    }

    /**
     * Performs a comparison between a string and the current string value of the String object.
     * @param string|String $str The string to compare to.
     * @param bool $caseInsensitive Whether to perform a case insensitive comparison or not.
     * @param int $length If set to higher than 0 will perform a comparison of the first $length characters.
     * @return int Returns 0 it both strings are equal, 1 if the current string value is greater than the other and -1 otherwhise.
     */
    public function compare($str, $caseInsensitive = false, $length = -1)
    {
        $strToCompare = ($str instanceof String) ? $str->toString() : $str;

        if ($length > 0) {
            return ($caseInsensitive) ? 
                strncasecmp($this->str, $strToCompare, $length) : 
                strncmp($this->str, $strToCompare, $length);
        } else {
            return ($caseInsensitive) ? 
                strcasecmp($this->str, $strToCompare, $length) : 
                strcmp($this->str, $strToCompare, $length);
        }
    }

    /**
     * Strip HTML and PHP tags from a string.
     * @link http://www.php.net/manual/en/function.strip-tags.php PHP documentation for strip_tags.
     * @param string $allowableTags You can use this optional parameter to specify tags which should not be stripped.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function stripTags($allowableTags = '', $alter = false)
    {
        if ($alter)
            $this->str = strip_tags($this->str, $allowableTags);
        else
            return strip_tags($this->str, $allowableTags);
    }

    /**
     * Un-quote string quoted with addcslashes.
     * @link http://www.php.net/manual/en/function.stripcslashes.php PHP documentation for stripcslashes.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function stripCSlashes($alter = false)
    {
        if ($alter)
            $this->str = stripcslashes($this->str);
        else
            return stripcslashes($this->str);
    }

    /**
     * Un-quote a quoted string.
     * @link http://www.php.net/manual/en/function.stripslashes.php PHP documentation for stripslashes.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function stripSlashes($alter = false)
    {
        if ($alter)
            $this->str = stripslashes($this->str);
        else
            return stripslashes($this->str);
    }

    /**
     * Returns the first position of a string found in the current string value.
     * @param string|String $needle The string to look for in the current string value.
     * @param int $offset If specified, search will start this number of characters counted from the beginning of the string. If the offset is negative it will be omitted.
     * @param bool $caseInsensitive Whether to perform a case insensitive search or not.
     * @return int The position of where the needle exists relative to the beginnning of the current string value.
     */
    public function indexOf($needle, $offset = 0, $caseInsensitive = false)
    {
        if ($offset < 0)
            $offset = 0;
        if ($needle instanceof String) {
            return ($caseInsensitive) ? 
                stripos($this->str, $needle->toString(), $offset) : 
                strpos($this->str, $needle->toString(), $offset);
        } else {
            return ($caseInsensitive) ? 
                stripos($this->str, $needle, $offset) : 
                strpos($this->str, $needle, $offset);
        }
    }

    /**
     * Returns the last position of a string found in the current string value.
     * @param string|String $needle The string to look for in the current string value.
     * @param int $offset If specified, search will start this number of characters counted from the beginning of the string. If the offset is negative it will be omitted.
     * @param bool $caseInsensitive Whether to perform a case insensitive search or not.
     * @return int The position of where the last occurrence of needle was found relative to the beginnning of the current string value.
     */
    public function lastIndexOf($needle, $offset = 0, $caseInsensitive = false)
    {
        if ($needle instanceof String) {
            return ($caseInsensitive) ? 
                strripos($this->str, $needle->toString(), $offset) : 
                strrpos($this->str, $needle->toString(), $offset);
        } else {
            return ($caseInsensitive) ? 
                strripos($this->str, $needle, $offset) : 
                strrpos($this->str, $needle, $offset);
        }
    }

    /**
     * Returns the length of the current string value.
     * @return int The length of the string.
     */
    public function length()
    {
        return strlen($this->str);
    }

    /**
     * Make the string lowercase.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function toLower($alter = false)
    {
        if ($alter)
            $this->str = strtolower($this->str);
        else
            return strtolower($this->str);
    }

    /**
     * Make the string uppercase.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function toUpper($alter = false)
    {
        if ($alter)
            $this->str = strtoupper($this->str);
        else
            return strtoupper($this->str);
    }

    /**
     * Make a string's first character uppercase.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function upperFirst($alter = false)
    {
        if ($alter)
            $this->str = ucfirst($this->str);
        else
            return ucfirst($this->str);
    }

    /**
     * Uppercase the first character of each word in the string.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function upperCaseWords($alter = false)
    {
        if ($alter)
            $this->str = ucwords($this->str);
        else
            return ucwords($this->str);
    }

    /**
     * Reverses the string.
     * @param bool $alter If true the current string value will be altered, otherwhise it will be returned but it won't change.
     * @return string If $alter is false.
     */
    public function reverse($alter = false)
    {
        if ($alter)
            $this->str = strrev($this->str);
        else
            return strrev($this->str);
    }

    /**
     * Tokenizes the string.
     * @link http://www.php.net/manual/en/function.strtok.php PHP documentation for strtok.
     * @param string $token The delimiter used when splitting up the string.
     * @return string A string token.
     */
    public function tokenize($token)
    {
        return strtok($this->str, $token);
    }

    /**
     * Returns part of the string.
     * @link http://www.php.net/manual/en/function.substr.php PHP documentation for substr.
     * @param int $from Start position of the part that will be returned.
     * @param int $to If specified, the end position of the part that will be returned.
     * @param int $length If specified and $to is not specified, the length of characters to return from the start position set by $from.
     * @return string The desired part of the string.
     */
    public function substring($from, $to = false, $length = false)
    {
        if (is_int($to))
            return substr($this->str, $from, $to - $from);
        elseif (is_int($length))
            return substr($this->str, $from, $length);
        else
            return substr($this->str, $from);
    }

    /**
     * Checks if a string is contained in the current string value.
     * @param string $needle The string to look for in the current string value.
     * @param bool $caseInsensitive Whether to perform a case insensitive search or not.
     * @return bool The existance of the needle in the current string.
     */
    public function contains($needle, $caseInsensitive = false)
    {
        return $this->indexOf((string) $needle, 0, $caseInsensitive) !== false;
    }

    /**
     * Checks if two strings are equal.
     * @param string|String $str The string to compare to.
     * @return bool The equalness of the two strings.
     */
    public function equals($str)
    {
        return ($str instanceof String) ? 
            $this->toString() === $str->toString() :
            $this->toString() === $str;
    }

    /**
     * Clones the current string value.
     * @return String The cloned string.
     */
    public function cloneString() {
        return new String($this->toString());
    }

    /**
     * Returns the primitive string value of the String object.
     * @return string The primitive string value.
     */
    public function toString()
    {
        return $this->str;
    }

    /**
     * Returns the primitive string value of the String object.
     * @return string The primitive string value.
     */
    public function __toString()
    {
        return $this->toString();
    }
}
